# Some utility function for use in installing products in Hudson

# Finds the directory that contains the Update Site
findUS() {
	local startDir="$1"
#	local temp="$WORKSPACE/../../$US_JOB_NAME/builds/$US_BUILD_ID/archive"
#	if [[ ! -d "$temp" ]]; then
#  		temp="$WORKSPACE/../../$US_JOB_NAME/$US_BUILD_ID/archive"
#	fi
	REPOS=file:/$(find "$startDir" -name site.p2 -type d | head -1)
}

# Installs an IU
installIU() {
	local iu="$1"
	local repos="$2"
	local dest="$OUTPUT_DIR/$3"
	local os="$4"
	local ws="$5"
	local arch="$6"

	$DIRECTOR -repository "$repos" -installIU "$iu" -destination "$dest" \
		-profile KNIMEProfile -profileProperties org.eclipse.update.install.features=true \
		-bundlepool "$dest" -p2.os $os -p2.ws $ws -p2.arch $arch -roaming		
}


# Creates ZIP archives from installed products, optionally naming them with the
# build date if this is a nightly build
createArchives() {
	local dest="$1"
	local os="$2"
	local ws="$3"
	local arch="$4"

	pushd "$OUTPUT_DIR"
	local version=$(find "$dest/features" -name 'org.knime.*product*' | sort | head -1)
    if [[ -z "$version" ]]; then
       version=$(find "$dest/features" -name 'org.knime.features.base*' | sort | head -1)
    fi
  	version=${version##*/} # strip directories
	version=${version##*_} # strip feature name
	version=${version%.*} # strip last version part
	
	if [[ "$NIGHTLY" == "true" ]]; then
		version="${version}_${BUILD_ID}"
	fi
	mv "$dest" "${dest}_${version}"

	
	if [[ "$os" == "win32" ]]; then
		zip -qry9 "${dest}_${version}.$os.$ws.$arch.zip" "${dest}_${version}"
		md5sum -b "${dest}_${version}.$os.$ws.$arch.zip" > "${dest}_${version}.$os.$ws.$arch.zip.md5"

		7z a "${dest}_${version}.$os.$ws.$arch.7z" "${dest}_${version}" > /dev/null
		cat "$WORKSPACE/org.knime.build/files/7z.sfx" \
			"${dest}_${version}.$os.$ws.$arch.7z" > "${dest}_${version}.$os.$ws.$arch.exe"
		md5sum -b  "${dest}_${version}.$os.$ws.$arch.exe" > "${dest}_${version}.$os.$ws.$arch.exe.md5"
		rm "${dest}_${version}.$os.$ws.$arch.7z"
	elif [[ "$os" == "linux" ]]; then
		tar -c "${dest}_${version}" | gzip -9 > "${dest}_${version}.$os.$ws.$arch.tar.gz"
		md5sum -b "${dest}_${version}.$os.$ws.$arch.tar.gz" > "${dest}_${version}.$os.$ws.$arch.tar.gz.md5"	
	elif [[ "$os" == "macosx" ]]; then
		# TODO create a dmg
		zip -qry9 "${dest}_${version}.$os.$ws.$arch.zip" "${dest}_${version}"
		md5sum -b "${dest}_${version}.$os.$ws.$arch.zip" > "${dest}_${version}.$os.$ws.$arch.zip.md5"		
	else
		zip -qry9 "${dest}_${version}.$os.$ws.$arch.zip" "${dest}_${version}"
		md5sum -b "${dest}_${version}.$os.$ws.$arch.zip" > "${dest}_${version}.$os.$ws.$arch.zip.md5"	
	fi

	rm -rf "${dest}_${version}"
	popd
}

# Runs a command with a timeout. If it has not finished until then it is killed.
# First argument is the timeout, all following arguments are the command and
# its arguments.
runWithWatchdog() {
	local shellAttributes=$-
	set +x # disable tracing, because in Hudson this fills the log file
	local timeout="$1"
	shift

	"$@" &
	WAIT_FOR_PID=$!
	while [[ $timeout -gt 0 ]]; do		
		if ! $(ps -p $WAIT_FOR_PID h > /dev/null); then
			set -$shellAttributes
			return 0
		fi
		sleep 1
		timeout=$((timeout - 1))
	done
	echo "Timeout occurred, killing job"
	if [[ "$(ps -o comm -p $WAIT_FOR_PID --no-header)" == "java" ]]; then
	   jstack -l $WAIT_FOR_PID
	fi
	set -$shellAttributes
	kill $WAIT_FOR_PID
	return 1
}


sendTestingMails() {
    local testResultDir="$1"
    
    local summary=$( /bin/ls -1t "$testResultDir/with_assertions/"_Summary_*.txt | head -1 )
    if [[ -n "$summary" ]]; then
        local recipients=""
        for i in $( grep "failed" $summary | grep -Eo '[^@ ]+@[^)]+' | sort | uniq ); do
            recipients="$i $recipients";
        done

        if [[ -n "$recipients" ]]; then
            recipients="$ALWAYS_ERROR $recipients"
        fi
        recipients="$ALWAYS $recipients"

        echo "Detailed results in $BUILD_URL" > /tmp/mail.$$
        echo >> /tmp/mail.$$
        echo >> /tmp/mail.$$
        cat $summary >> /tmp/mail.$$
        dos2unix < /tmp/mail.$$ | \
            mail -s "KNIME Regression Tests $( date +%Y-%m-%d ) (with assertions)" \
                -r "The master of KNIME <noreply@knime.org>" $recipients
        rm /tmp/mail.$$
    fi

    summary=$( /bin/ls -1 "$testResultDir/without_assertions/"_Summary_*.txt )
    if [[ -n "$summary" ]]; then
        recipients=""
        for i in "$testResultDir/without_assertions/"[^_]*.txt; do
            local shortName=${i##*/}
            shortName=${shortName%_?????_??_??_??.txt}
            for f in "$testResultDir/with_assertions/"${SHORTNAME}*.txt; do
                if [[ ! -f $f ]]; then
                    recipients="$( head -1 $i | sed -e 's/[\r\n]//g' ) $recipients"
                fi
            done
        done

        if [[ -n "$recipients" ]]; then
            recipients="$ALWAYS_ERROR $recipients"
        fi
        recipients="$ALWAYS $recipients"

        echo "Detailed results in $BUILD_URL" > /tmp/mail.$$
        echo >> /tmp/mail.$$
        echo >> /tmp/mail.$$
        cat $summary >> /tmp/mail.$$
        dos2unix < /tmp/mail.$$ | \
            mail -s "KNIME Regression Tests $( date +%Y-%m-%d ) (without assertions)" \
                -r "The master of KNIME <noreply@knime.org>" $recipients
        rm /tmp/mail.$$
    fi
}


shellAttributes=$-
set +x # disable tracing, because in Hudson this fills the log file

# An somewhat complicated way to read a property from a Java properties file
# from the shell. Leading spaces in multiline properties are stripped.
#
# First argument to readProperty is the property file, the second the property
# name. 
__PERL_CODE__=$(cat << '__END__'
if (@ARGV != 2) {
        print STDERR "Usage: readProperty file property-name\n";
        exit 1;
}

my $file = $ARGV[0];
my $property = $ARGV[1];

my $value;
open (IP, "< $file") || die "Cannot open $file for reading";
while (my $line = <IP>) {
        chomp($line);
        if ($line =~ /^$property=(.*)/) { 
                $value = $1;
        } elsif ($value) {
        		$line =~ s/^\s+//;
                $value .= $line;
        }
        if ($value) {
                last if($value !~ /\\$/);
                $value =~ s/\\$//;
        }
}
close IP;

print $value || "";
print "\n";
__END__
)

alias readProperty="perl -e \"\$__PERL_CODE__\""
set -$shellAttributes;