/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2011
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * ---------------------------------------------------------------------
 *
 * History
 *   26.04.2011 (meinl): created
 */
package org.knime.build.actors;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

import org.eclipse.buckminster.core.actor.AbstractActor;
import org.eclipse.buckminster.core.actor.IActionContext;
import org.eclipse.buckminster.core.cspec.PathGroup;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.knime.build.tools.SplashTool;

/**
 *
 * @author Thorsten Meinl, University of Konstanz
 */
public class BuildDateActor extends AbstractActor {
    private static final DateFormat DATE_FORMAT = new SimpleDateFormat(
            "MMMM dd, yyyy");

    @Override
    protected IStatus internalPerform(final IActionContext ctx,
            final IProgressMonitor monitor) throws CoreException {
        File outputDir =
                ctx.getAction()
                        .getExpandedBase(ctx.getAction().getProductBase(),
                                ctx.getProperties()).toFile();
        outputDir.mkdirs();

        String buildDate =
                "Nightly-Build " + DATE_FORMAT.format(new Date());

        for (Map.Entry<String, PathGroup[]> e : ctx.getNamedPathGroupArrays()
                .entrySet()) {
            if (e.getKey().endsWith(".knime.constants.java")) {
                for (PathGroup pg : e.getValue()) {
                    IPath base = pg.getBase();
                    for (IPath path : pg.getPaths()) {
                        IPath constantsFile = base.append(path);
                        try {
                            SplashTool.changeConstantsDate(
                                    constantsFile.toFile(), buildDate);
                        } catch (Exception ex) {
                            Status status =
                                    new Status(IStatus.ERROR,
                                            "org.knime.build",
                                            "Could not change splash date", ex);
                            throw new CoreException(status);
                        }
                    }
                }
            }
        }

        return Status.OK_STATUS;
    }
}