/* ------------------------------------------------------------------
 * This source code, its documentation and all appendant files
 * are protected by copyright law. All rights reserved.
 *
 * Copyright, 2003 - 2011
 * University of Konstanz, Germany
 * Chair for Bioinformatics and Information Mining (Prof. M. Berthold)
 * and KNIME GmbH, Konstanz, Germany
 *
 * You may not modify, publish, transmit, transfer or sell, reproduce,
 * create derivative works from, distribute, perform, display, or in
 * any way exploit any of the content, in whole or in part, except as
 * otherwise expressly permitted in writing by the copyright owner or
 * as specified in the license file distributed with this product.
 *
 * If you have any questions please contact the copyright holder:
 * website: www.knime.org
 * email: contact@knime.org
 * ---------------------------------------------------------------------
 *
 */
package org.knime.build.actors;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

import org.eclipse.buckminster.core.actor.AbstractActor;
import org.eclipse.buckminster.core.actor.IActionContext;
import org.eclipse.buckminster.core.cspec.PathGroup;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.knime.build.tools.SplashTool;

public class SplashDateActor extends AbstractActor {
    private static final DateFormat DATE_FORMAT = new SimpleDateFormat(
            "yyyy-MM-dd HH:mm");

    @Override
    protected IStatus internalPerform(final IActionContext ctx,
            final IProgressMonitor monitor) throws CoreException {
        File outputDir =
                ctx.getAction()
                        .getExpandedBase(ctx.getAction().getProductBase(),
                                ctx.getProperties()).toFile();
        outputDir.mkdirs();

        String buildDate =
                "(Nightly-Build " + DATE_FORMAT.format(new Date()) + ")";

        for (Map.Entry<String, PathGroup[]> e : ctx.getNamedPathGroupArrays()
                .entrySet()) {
            if (e.getKey().endsWith(".knime.splashFile.svg")) {
                for (PathGroup pg : e.getValue()) {
                    IPath base = pg.getBase();
                    for (IPath path : pg.getPaths()) {
                        IPath splashFile = base.append(path);
                        try {
                            SplashTool.changeSplashDate(splashFile.toFile(),
                                    outputDir, buildDate);
                        } catch (Exception ex) {
                            Status status =
                                    new Status(IStatus.ERROR,
                                            "org.knime.build",
                                            "Could not change splash date", ex);
                            throw new CoreException(status);
                        }
                    }
                }
            }
        }

        return Status.OK_STATUS;
    }
}
