/* ------------------------------------------------------------------
 * This source code, its documentation and all appendant files
 * are protected by copyright law. All rights reserved.
 *
 * Copyright, 2003 - 2011
 * University of Konstanz, Germany
 * Chair for Bioinformatics and Information Mining (Prof. M. Berthold)
 * and KNIME GmbH, Konstanz, Germany
 *
 * You may not modify, publish, transmit, transfer or sell, reproduce,
 * create derivative works from, distribute, perform, display, or in
 * any way exploit any of the content, in whole or in part, except as
 * otherwise expressly permitted in writing by the copyright owner or
 * as specified in the license file distributed with this product.
 *
 * If you have any questions please contact the copyright holder:
 * website: www.knime.org
 * email: contact@knime.org
 * ---------------------------------------------------------------------
 *
 * History
 *   06.08.2010 (meinl): created
 */
package org.knime.build.tools;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Program that sets the build date in the splash screen and in KNIMEConstants.
 *
 * @author Thorsten Meinl, University of Konstanz
 */
public final class SetBuildDate {
    private SetBuildDate() {
    }

    private static void usage() {
        System.out.print("Usage: " + SetBuildDate.class.getCanonicalName());
        System.out.println(" splashFile corePluginDir [date]");
        System.out.println("If no date is given the current date is used.");
    }

    /**
     * @param args
     */
    public static void main(final String[] args) throws Exception {
        if (args.length < 2) {
            usage();
            System.exit(1);
        }

        File splashFile = new File(args[0]);
        if (!splashFile.isFile()) {
            throw new IllegalArgumentException("Splash file '" + splashFile
                    + "' does not exist");
        }

        File corePluginDir = new File(args[1]);
        if (!corePluginDir.isDirectory()) {
            throw new IllegalArgumentException("Core plugin directory '"
                    + corePluginDir + "' does not exist");
        }

        File knimeConstants =
                new File(corePluginDir,
                        "src/org/knime/core/node/KNIMEConstants.java");
        if (!knimeConstants.isFile()) {
            throw new IllegalArgumentException(
                    "KNIMEConstants.java not found in core plugin");
        }

        String date;
        if (args.length > 2) {
            date = args[2];
        } else {
            // December 06, 2010
            date = new SimpleDateFormat("MMMM dd, yyyy").format(new Date());
        }

        System.out.println("Setting build date in "
                + splashFile.getAbsolutePath());
        SplashTool.changeSplashDate(splashFile, splashFile.getParentFile(),
                "(Build " + date + ")");

        System.out.println("Setting build date in "
                + knimeConstants.getAbsolutePath());
        SplashTool.changeConstantsDate(knimeConstants, date);
    }
}
