/* ------------------------------------------------------------------
 * This source code, its documentation and all appendant files
 * are protected by copyright law. All rights reserved.
 *
 * Copyright, 2003 - 2011
 * University of Konstanz, Germany
 * Chair for Bioinformatics and Information Mining (Prof. M. Berthold)
 * and KNIME GmbH, Konstanz, Germany
 *
 * You may not modify, publish, transmit, transfer or sell, reproduce,
 * create derivative works from, distribute, perform, display, or in
 * any way exploit any of the content, in whole or in part, except as
 * otherwise expressly permitted in writing by the copyright owner or
 * as specified in the license file distributed with this product.
 *
 * If you have any questions please contact the copyright holder:
 * website: www.knime.org
 * email: contact@knime.org
 * ---------------------------------------------------------------------
 *
 */
package org.knime.build.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.xpath.XPathFactory;

public final class XMLHelper {
    static final DocumentBuilder parser;

    static final Transformer serializer;

    static final XPathFactory xPathFactory;

    static final String LINE_SEP = System.getProperty("line.separator");

    static {
        try {
            parser = DocumentBuilderFactory.newInstance().newDocumentBuilder();

            TransformerFactory tf = TransformerFactory.newInstance();
            serializer = tf.newTransformer();
            serializer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
            serializer.setOutputProperty(OutputKeys.INDENT, "yes");

            xPathFactory = XPathFactory.newInstance();
        } catch (Exception ex) {
            if (ex instanceof RuntimeException) {
                throw (RuntimeException)ex;
            } else {
                throw new RuntimeException(ex);
            }
        }
    }

    private XMLHelper() {
    }

    public static void copy(final File file, final File destination)
            throws IOException {
        byte[] cache = new byte[8192];
        OutputStream copyOutStream = new FileOutputStream(destination);
        InputStream copyInStream = new FileInputStream(file);
        int read;
        while ((read = copyInStream.read(cache, 0, cache.length)) > 0) {
            copyOutStream.write(cache, 0, read);
        }
        copyOutStream.close();
        copyInStream.close();
    }

}
